package Stanford::DataAccessRequest;

use strict;
use warnings;
use autodie;

use Carp;
use English;

sub new {
    my $class = shift;
    my ($args) = @_;

    my $self = {
        'debug'     => $args->{'debug'} ? 1 : 0,
        'dbh'       => $args->{'dbh'},
        'host'      => $args->{'host'},
        'db_name'   => $args->{'db_name'},
        'username'  => $args->{'username'},
        'password'  => $args->{'password'},
        'config'    => {},
    };

    bless $self, $class;

    return $self;
}

sub debug {
    my $self = shift;

    if ($self->{'debug'}) {
        return 1;
    } else {
        return 0;
    }
}

sub dbh {
    my $self = shift;
    my ($dbh) = @_ ;

    if ($dbh) {
        $self->{'dbh'} = $dbh;
    }

    return $self->{'dbh'};
}

sub set_config {
    my $self = shift;
    my ($config_option, $value) = @_ ;

    $self->{'config'}->{$config_option} = $value;

    return $self->get_config($config_option) ;
}

sub get_config {
    my $self = shift;
    my ($config_option) = @_ ;

    return $self->{'config'}->{$config_option};
}

sub get_configs {
    my $self = shift;

    return %{ $self->{'config'} } ;
}

sub dbg {
    my $self = shift;
    my ($msg) = @_ ;
    if ($self->debug()) {
        print "dbg: $msg\n";
    }
    return;
}

sub read_conf {
    my $self = shift;

    my $config_file = '/etc/data-access-request/config.ini';

    $self->dbg("reading configration file $config_file");
    my $cfg = Config::IniFiles->new( -file => $config_file, -fallback => 'main' );

    my @parameters = qw(
        dataowner_approval_url
        database_connect_file
        dataowner_from
        dataowner_notify_APPROVE
        dataowner_notify_AUDIT
        email_override
        production_mode
        ready_for_implementation_email_saml
        ready_for_implementation_email_ldap
    );

    foreach my $parameter (@parameters) {
        my $value = $cfg->val('main', $parameter);

        if ($parameter eq 'production_mode') {
            if ($value =~ m{true}ixsm) {
                $value = 1;
            } elsif ($value =~ m{false}ixsm) {
                $value = 0;
            } else {
                croak "do not understand $value for production_mode";
            }
        } else {
            $value = trim($value);
        }

        $self->set_config($parameter, $value);
    }

    return;
}

sub database_connect {
    my $self = shift;

    $self->dbg('entering database_connect routine');
    my $database_connect_file = $self->get_config('database_connect_file');

    if (!$database_connect_file) {
        croak 'no database connect file found';
    }

    my $cfg = Config::IniFiles->new( -file => $database_connect_file, -fallback => 'main' );

    my $host     = $cfg->val('main', 'host');
    my $db_name  = $cfg->val('main', 'db_name');
    my $username = $cfg->val('main', 'username');
    my $password = $cfg->val('main', 'password');

    $self->dbg("host:     $host");
    $self->dbg("db_name:  $db_name");
    $self->dbg("username: $username");
    $self->dbg('password: REDACTED');

    my $dbh = DBI->connect("DBI:mysql:host=$host;database=$db_name;"
                           . 'port=3306',
                           $username,
                           $password,
                           {RaiseError => 1})
        or croak "unable to connect to database: $DBI::errstr\n";
    $self->dbg("connected to database $host:$db_name");
    $self->dbh($dbh);

    return $self->dbh();
}

sub trim {
    my ($s) = @_ ;

    if (!defined($s)) { return $s }

    $s =~ s{^\s*}{}xs ;  ## no critic (RequireLineBoundaryMatching) ;
    $s =~ s{\s*$}{}xs ;  ## no critic (RequireLineBoundaryMatching) ;

    return $s ;
}

# ----------------------------------------------------------
# get unique number

sub next_number {
    my $self = shift;

    my ($a_id) = @_;

    if (!$a_id) {
        die "Invalid ID for next_number";
    }

    my $dbh = $self->dbh();

    # look to see if we have one at all
    my $sel = "SELECT next_number FROM next_number WHERE id='$a_id' ";
    my $sth = $dbh->prepare($sel);

    $sth->execute();

    my $row = $sth->fetchrow_hashref();
    my $new_nn = 0;
    if ($row->{next_number}) {

        # increment the next_number (which is really the last number used)
        my $cmd = "UPDATE next_number SET ";
        $cmd .= "next_number = LAST_INSERT_ID(next_number+1), ";
        $cmd .= "date_last_maint = NOW() ";
        $cmd .= "WHERE id='$a_id' ";
        my $sth = $dbh->prepare($cmd)
            || die "next_number prepare error.  Problem SQL:$cmd";
        $sth->execute()
            || die "next_number execute failed.  Problem SQL:$cmd";

        # retreive the resulting value
        my $sel = "SELECT LAST_INSERT_ID() next_number ";
        $sth = $dbh->prepare ($sel)
            || die "next_number  prepare failed.  Problem SQL:$sel";
        $sth->execute()
            || die "next_number execute failed.  Problem SQL:$sel";
        my $nnrow = $sth->fetchrow_hashref();
        $new_nn = $nnrow->{next_number};

    } else {

        # create a new one
        $new_nn = 1;
        my $cmd = "INSERT INTO next_number SET ";
        $cmd .= "id='$a_id', ";
        $cmd .= "next_number = $new_nn, ";
        $cmd .= "date_last_maint = NOW(), ";
        $cmd .= "date_added = NOW() ";
        my $sth = $dbh->prepare ($cmd)
            || die "next_number prepare failed.  Problem SQL:$cmd";
        $sth->execute()
            || die "next_number execute failed.  Problem SQL:$cmd";

    }
    return $new_nn;
}

sub do_log_change {
    my $self = shift;

    my %parm = @_;

    my $r = 'do_log_change';
    my $id = $self->next_number('DATAOWNER-MAINT');

    my $dbh = $self->dbh();

    my $cmd = "INSERT INTO dataowner_log ";
    $cmd .= "SET ";
    $cmd .= " record_id  = ?, ";
    $cmd .= " update_id  = ?, ";
    $cmd .= " action     = ?, ";
    $cmd .= " table_name = ?, ";
    $cmd .= " key_fields = ?, ";
    $cmd .= " key_values = ?, ";
    $cmd .= " field_name = ?, ";
    $cmd .= " old_value  = ?, ";
    $cmd .= " new_value  = ?, ";
    $cmd .= " uid        = ?, ";
    $cmd .= " date_added = NOW() ";

    eval {
        my $sth = $dbh->prepare($cmd);
        $sth->execute($id,
                      $parm{updateID},
                      $parm{action},
                      $parm{table},
                      $parm{keyFields},
                      $parm{keyValues},
                      $parm{fieldName},
                      $parm{oldValue},
                      $parm{newValue},
                      $parm{uid});
    };
    if ($@) {
        my $msg = "$r error: ($@)\n" . "Problem SQL: ($cmd)\n";
        die $msg;
    }

    return;
}

1;
