package Stanford::DataAccessRequest::MailObject;

use strict;
use warnings;
use autodie;

use Mail::Sendmail;
use Carp;
use Sys::Hostname;

use Stanford::DataAccessRequest;

my $SMTP = 'smtp-unencrypted.stanford.edu';

sub new {
    my $class = shift;

    my $parameters_href = shift ;
    my %parameters_href = %{ $parameters_href } ;

    my $self = {
        'from'     => $parameters_href->{'from'},
        'to'       => $parameters_href->{'to'},
        'subject'  => $parameters_href->{'subject'},
        'cc'       => $parameters_href->{'cc'},
        'bcc'      => $parameters_href->{'bcc'},
        'subject'  => $parameters_href->{'subject'},
        'body'     => $parameters_href->{'body'},
        'smtp'     => $parameters_href->{'smtp'},
        'override' => $parameters_href->{'override'},
        'production_mode'
                   => $parameters_href->{'production_mode'},
    };

    bless $self, $class;
    return $self;
}

sub to_string {
    my $self = shift;

    my $body     = $self->{'body'};
    my $subject  = $self->{'subject'};
    my $from     = $self->{'from'};
    my $cc       = $self->{'cc'};
    my $bcc      = $self->{'bcc'};
    my $override = $self->{'override'};
    my $production_mode
                 = $self->{'production_mode'};

    if (!$production_mode) {
        $subject = "[TESTING MODE] $subject";
	my $hostname = hostname();
        $subject .= " [$hostname]";
    }

    my $to_formatted;
    if ($override) {
	$to_formatted = $override;
    } else {
	# Remember that $to is an array, so we need to expand and format it.
	$to_formatted = $self->to_addresses_to_string();
    }

    my $CC  = q{};
    my $BCC = q{};
    if (!$override) {
	$CC  = q{};
	$BCC = q{};
	if ($cc) {
	    $CC = "Cc: $cc\n";
	}
	if ($bcc) {
	    $BCC = "Bcc: $bcc\n";
	}
    }

    my $body_formatted = $self->format_body();

    return <<"EOS";
From:    $from
To:      $to_formatted
Subject: $subject
$CC$BCC
$body_formatted
EOS
}

sub to_addresses_to_string {
    my $self = shift;
    my ($separator) = @_ ;

    if (!defined($separator)) {
        $separator = q{,};
    }

    # Remember that $to is an array, so we need to expand and format it.
    my @to_addresses = @{ $self->{'to'} };
    my @tos_formatted = ();
    foreach my $to_address (@to_addresses) {
        my ($fullname, $email) = @{ $to_address } ;
        $fullname = Stanford::DataAccessRequest::trim($fullname);
        $email    = Stanford::DataAccessRequest::trim($email);
        if (!$fullname) {
            push(@tos_formatted, $email);
        } else {
            push(@tos_formatted, "$fullname <$email>");
        }
    }

    my $to_formatted = join($separator, @tos_formatted);
    return $to_formatted ;
}

# In case we override the to address we need to format the body first.
sub format_body {
    my $self = shift;

    my $body     = $self->{'body'};
    my $override = $self->{'override'};

    my $body_formatted;
    if ($override) {
        my $to_addresses_formatted = $self->to_addresses_to_string();
        my @to_addresses = split(/,/xsm, $to_addresses_formatted) ;
        my $to_addresses_lines = join("\n", @to_addresses);
        $body_formatted = "[NOTE: all recipient addresses have been overridden. Here are the original e-mail addresses:\n"
	    . $to_addresses_lines . "]\n\n" . $body ;
    } else {
	$body_formatted = $body;
    }

    return $body_formatted;
}

sub send_mail {
    my $self = shift;

    my $body     = $self->{'body'};
    my $subject  = $self->{'subject'};
    my $from     = $self->{'from'};
    my $to       = $self->{'to'};
    my $cc       = $self->{'cc'};
    my $bcc      = $self->{'bcc'};
    my $smtp     = $self->{'smtp'};
    my $override = $self->{'override'};

    if (!$body) {
        croak 'send_message: missing body';
    }
    if (!$subject) {
        croak 'send_message: missing subject';
    }

    if (!$self->{'production_mode'}) {
        $subject = "[TESTING MODE] $subject";
	my $hostname = hostname();
	$subject .= " [$hostname]";
    }

    if (!$smtp) {
        $smtp = $SMTP;
    }

    my %mail = ();

    my $to_string;
    if ($override) {
	$to_string = $override;
    } else {
        # We only set Cc and Bcc if we are NOT overriding the e-mail address.
	$to_string   = $self->to_addresses_to_string();
        $mail{'Cc'}  = $cc;
        $mail{'Bcc'} = $bcc;
    }

    %mail = (
        'To'      => $to_string,
        'Subject' => $subject,
        'From'    => $from,
        'Message' => $self->format_body(),
        'Smtp'    => $smtp,
        );

    ## no critic (Variables::ProhibitPackageVars);
    sendmail(%mail) or croak $Mail::Sendmail::error;
    return;
}


1;
